#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Author:      thepoy
# @Email:       thepoy@163.com
# @File Name:   __main__.py
# @Created At:  2021-02-08 15:47:06
# @Modified At: 2023-02-21 12:43:38
# @Modified By: thepoy

import up2b

up2b.run_main()
